@@ -63,6 +63,7 @@ typedef int (*jerasure_make_decoding_bitmatrix_func)
     (int, int, int, int *, int *, int *, int *);
 typedef void (*jerasure_bitmatrix_dotprod_func)
     (int, int, int *, int *, int,char **, char **, int, int);
+typedef void (*galois_uninit_field_func)(int);
 
 /*
  * ToDo (KMG): Should we make this a parameter, or is that
@@ -76,6 +77,9 @@ struct jerasure_rs_cauchy_descriptor {
     jerasure_matrix_to_bitmatrix_func jerasure_matrix_to_bitmatrix;
     jerasure_smart_bitmatrix_to_schedule_func jerasure_smart_bitmatrix_to_schedule;
 
+    /* calls required for free */
+    galois_uninit_field_func galois_uninit_field;
+
     /* calls required for encode */
     jerasure_bitmatrix_encode_func jerasure_bitmatrix_encode;
                             
@@ -273,6 +277,7 @@ static void * jerasure_rs_cauchy_init(struct ec_backend_args *args,
         cauchy_original_coding_matrix_func  initp;
         jerasure_matrix_to_bitmatrix_func matrixtobitmatrixp;
         jerasure_smart_bitmatrix_to_schedule_func matrixschedulep;
+        galois_uninit_field_func uninitp;
         jerasure_bitmatrix_encode_func encodep;
         jerasure_bitmatrix_decode_func decodep;
         jerasure_erasures_to_erased_func erasedp; 
@@ -338,6 +343,13 @@ static void * jerasure_rs_cauchy_init(struct ec_backend_args *args,
         goto error; 
     } 
 
+    func_handle.vptr = NULL;
+    func_handle.vptr = dlsym(backend_sohandle, "galois_uninit_field");
+    desc->galois_uninit_field = func_handle.uninitp;
+    if (NULL == desc->galois_uninit_field) {
+        goto error;
+    }
+
     /* setup the Cauchy matrices and schedules */
     desc->matrix = desc->cauchy_original_coding_matrix(k, m, w);
     if (NULL == desc->matrix) {
@@ -345,17 +357,21 @@ static void * jerasure_rs_cauchy_init(struct ec_backend_args *args,
     }
     desc->bitmatrix = desc->jerasure_matrix_to_bitmatrix(k, m, w, desc->matrix);
     if (NULL == desc->bitmatrix) {
-        goto error;
+        goto bitmatrix_error;
     }
     desc->schedule = desc->jerasure_smart_bitmatrix_to_schedule(k, m, w, desc->bitmatrix);
     if (NULL == desc->schedule) {
-        goto error;
+        goto schedule_error;
     }
 
     return desc;
 
+schedule_error:
+    free(desc->bitmatrix);
+bitmatrix_error:
+    free(desc->matrix);
 error:
-    free_rs_cauchy_desc(desc);
+    free(desc);
     return NULL;
 }
 
@@ -385,6 +401,15 @@ static void free_rs_cauchy_desc(
         return;
     }
 
+    /*
+     * jerasure allocates some internal data structures for caching
+     * fields. It will allocate one for w, and if we do anything that
+     * needs to xor a region >= 16 bytes, it will also allocate one
+     * for 32. Fortunately we can safely uninit any value; if it
+     * wasn't inited it will be ignored.
+     */
+    jerasure_desc->galois_uninit_field(jerasure_desc->w);
+    jerasure_desc->galois_uninit_field(32);
     free(jerasure_desc->matrix);
     free(jerasure_desc->bitmatrix);
 
@@ -426,7 +451,6 @@ static bool jerasure_rs_cauchy_is_compatible_with(uint32_t version) {
     return version == backend_jerasure_rs_cauchy.ec_backend_version;
 }
 
-
 struct ec_backend_op_stubs jerasure_rs_cauchy_op_stubs = {
     .INIT                       = jerasure_rs_cauchy_init,
     .EXIT                       = jerasure_rs_cauchy_exit,
@@ -436,7 +460,8 @@ struct ec_backend_op_stubs jerasure_rs_cauchy_op_stubs = {
     .RECONSTRUCT                = jerasure_rs_cauchy_reconstruct,
     .ELEMENTSIZE                = jerasure_rs_cauchy_element_size,
     .ISCOMPATIBLEWITH           = jerasure_rs_cauchy_is_compatible_with,
-
+    .GETMETADATASIZE            = get_backend_metadata_size_zero,
+    .GETENCODEOFFSET            = get_encode_offset_zero,
 };
 
 struct ec_backend_common backend_jerasure_rs_cauchy = {
@@ -445,7 +470,6 @@ struct ec_backend_common backend_jerasure_rs_cauchy = {
     .soname                     = JERASURE_RS_CAUCHY_SO_NAME,
     .soversion                  = JERASURE_RS_CAUCHY_LIB_VER_STR,
     .ops                        = &jerasure_rs_cauchy_op_stubs,
-    .backend_metadata_size      = 0,
     .ec_backend_version         = _VERSION(JERASURE_RS_CAUCHY_LIB_MAJOR,
                                            JERASURE_RS_CAUCHY_LIB_MINOR,
                                            JERASURE_RS_CAUCHY_LIB_REV),
