@@ -28,6 +28,7 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <zlib.h>
 #include "erasurecode_backend.h"
 #include "erasurecode_helpers.h"
 #include "erasurecode_helpers_ext.h"
@@ -474,7 +475,7 @@ inline int set_checksum(ec_checksum_type_t ct, char *buf, int blocksize)
 
     switch(header->meta.chksum_type) {
         case CHKSUM_CRC32:
-            header->meta.chksum[0] = crc32(0, data, blocksize);
+            header->meta.chksum[0] = crc32(0, (unsigned char *) data, blocksize);
             break;
         case CHKSUM_MD5:
             break;
@@ -500,46 +501,3 @@ inline uint32_t* get_chksum(char *buf)
 }
 
 /* ==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~== */
-
-#if LIBERASURECODE_VERSION >= _VERSION(1,2,0)
-inline int set_metadata_chksum(char *buf)
-{
-    fragment_header_t* header = (fragment_header_t*) buf;
-
-    assert(NULL != header);
-    if (header->magic != LIBERASURECODE_FRAG_HEADER_MAGIC) {
-        log_error("Invalid fragment header (set meta chksum)!\n");
-        return -1;
-    }
-
-    header->metadata_chksum = crc32(0, &header->meta,
-                                    sizeof(fragment_metadata_t));
-    return 0;
-}
-
-inline uint32_t* get_metadata_chksum(char *buf)
-{
-    fragment_header_t* header = (fragment_header_t*) buf;
-
-    assert(NULL != header);
-    if (header->magic != LIBERASURECODE_FRAG_HEADER_MAGIC) {
-        log_error("Invalid fragment header (get meta chksum)!");
-        return NULL;
-    }
-
-    return (uint32_t *) &header->metadata_chksum;
-}
-#else
-inline int set_metadata_chksum(char *buf)
-{
-    return 0;
-}
-
-inline uint32_t* get_metadata_chksum(char *buf)
-{
-    return (uint32_t *) 0;
-}
-#endif
-
-/* ==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~==~=*=~== */
-
